program Test
  use matmult
  use cdstep_mod
  use cgstep_mod
  use partan_mod
  use broyden_mod
  use conjgrad_mod
  use gmres_mod
  use solver_mod

  implicit none
  integer, parameter             :: n=4, m=5
  real, dimension (:,:), pointer :: a
  real, dimension (n)            :: x
  real, dimension (m)            :: y = (/ 3., 3., 5., 7., 9. /), r
  integer                        :: i

  allocate (a (m,n))

  a(1,:) = (/ 1., 1., 1., 0. /)
  a(2,:) = (/ 1., 2., 0., 0. /)
  a(3,:) = (/ 1., 3., 1., 0. /)
  a(4,:) = (/ 1., 4., 0., 1. /)
  a(5,:) = (/ 1., 5., 1., 1. /)

20 format('x  ',10f12.8)
21 format('res',10f12.8)

  call matmult_init (a)

  write (6,*) " Conjugate Gradients (Plane Search)"
  write (6,*) " ----------------------------------"

  do i = 1, n + 1
     call solver(matmult_lop, cgstep, niter = i, x = x, dat = y, res = r) 
     write(6,20) x
     write(6,21) r
     call cgstep_close()
  end do

  write (6,*) " Conjugate Directions"
  write (6,*) " --------------------"

  do i = 1, n + 1
     call cdstep_init ()
     call solver(matmult_lop, cdstep, niter = i, x = x, dat = y, res = r)
     write(6,20) x
     write(6,21) r
     call cdstep_close () 
  end do

  write (6,*) " Conjugate Gradients (Line Search)"
  write (6,*) " ---------------------------------"

  do i = 1, n + 1
     call solver(matmult_lop, partan, niter = i, x = x, dat = y, res = r)
     write(6,20) x
     write(6,21) r
     call partan_close ()
  end do

  write (6,*) " Conjugate Gradients (Traditional)"
  write (6,*) " ---------------------------------"

  do i = 1, n + 1
     call solver(matmult_lop, conjgrad, niter = i, &
     x = x, dat = y, res = r)
     write(6,20) x
     write(6,21) r
     call conjgrad_close ()
  end do

  write (6,*) " Conjugate Gradients (Traditional) with Ort"
  write (6,*) " ------------------------------------------"

  do i = 1, n + 1
     call solver(matmult_lop, conjgrad, niter = i, &
     x = x, dat = y, res = r, nort = i)
     write(6,20) x
     write(6,21) r
     call conjgrad_close ()
  end do


  write (6,*) " Generalized Minimum RESidual"
  write (6,*) " ----------------------------"

  do i = 1, n + 1
     call gmres_init ()
     call solver(matmult_lop, gmres, niter = i, &
     x = x, dat = y, res = r)
     write(6,20) x
     write(6,21) r
     call gmres_close ()
  end do

  write (6,*) " Broyden's method"
  write (6,*) " ----------------"

  do i = 1, 2*n + 1
     call broyden_init (i)
     call nonlin_solver(matmult_lop, broyden, niter = i, &
     x = x, dat = y, res = r)
     call broyden_close () 
     write(6,20) x
     write(6,21) r
  end do

  write (6,*) " LSQR"
  write (6,*) " ----"

  do i = 1, n + 1
     call lsqr(matmult_lop, niter = i, x = x, dat = y)
     write(6,20) x
  end do

  deallocate (a)
end program Test

