module pgrad
  use adj_mod
  use chain_mod
  use igrad
  use leakint

  implicit none
  integer,            private               :: n1, n2
  logical, parameter, private               :: T = .true.

contains
  subroutine pgrad_init (m1, m2)
    integer, intent (in) :: m1, m2
    n1 = m1 
    n2 = m2
  end subroutine pgrad_init

  function pgrad_op (adj, add, x, y) result (stat)
    integer                    :: stat
    logical, intent (in)       :: adj, add
    real, dimension (:)        :: x, y

    real, dimension (size (x)) :: tmp

    call chain (leakint_op, igrad_op, adj, add, x, y, tmp)
    stat = 0
  end function pgrad_op

  function pgrad2_op (adj, add, x, y) result (stat)
    integer              :: stat
    logical, intent (in) :: adj, add
    real, dimension (:)  :: x, y

    call adjnull (adj, add, x, y)
    call pgrad2_apply (adj, x, y)
    stat = 0
  end function pgrad2_op

  subroutine pgrad2_apply (adj, p, r)
    logical, intent (in)        :: adj
    real, dimension (n1, n2)    :: p
    real, dimension (n1, n2, 2) :: r

    integer                     :: i, stat

    do i = 1, n2
       stat = pgrad_op (adj, T, p (:,i), r (:, i, 1))
    end do

    do i = 1, n1
       stat = pgrad_op (adj, T, p (i,:), r (i, :, 2))
    end do

  end subroutine pgrad2_apply
end module pgrad









