module phasegrad
  use solver_mod
  use cgstep_mod
  use igrad2
  use pgrad
  use leakint

  implicit none

contains

  function grad2init (z) result (rt)
    complex, dimension (:,:), intent (in)           :: z
    real,    dimension (size (z, 1), size (z, 2),2) :: rt

    integer                                         :: n1,n2

    n1 = size (z, 1)
    n2 = size (z, 2)

    rt = 0.
    rt (:n1-1,:,1) = aimag (clog (z (2:,:)* conjg (z (:n1-1,:)))) 
    rt (:,:n2-1,2) = aimag (clog (z (:,2:)* conjg (z (:,:n2-1)))) 
  end function grad2init

  function pgrad2init (z) result (rt)
    complex, dimension (:,:), intent (in)           :: z
    real,    dimension (size (z, 1), size (z, 2),2) :: rt

    integer                                         :: i, n1,n2, stat

    n1 = size (z, 1)
    n2 = size (z, 2)

    rt = 0.
    do i = 1, n2
       stat = leakint_op (.false., .false., &
       aimag (clog (z (2:,i)* conjg (z (:n1-1,i)))),rt (:n1-1,i,1))
    end do

    do i = 1, n1
       stat = leakint_op (.false., .false., &
       aimag (clog (z (i,2:)* conjg (z (i,:n2-1)))),rt (i,:n2-1,2)) 
    end do
  end function pgrad2init

  subroutine unwrap (z, niter, q, hh, hmov)
    real,                       intent (in)     :: q
    integer,                    intent (in)     :: niter
    complex, dimension (:,:),   intent (in)     :: z
    real,    dimension (:,:),   intent (out)    :: hh
    real,    dimension (:,:,:), intent (out)    :: hmov

    real, dimension (size (z, 1),size (z, 2),2) :: rr

    if (q < 1.) then
       call leakint_init (q) ; rr = pgrad2init (z)
       call pgrad_init (size (z, 1), size (z, 2))
       call unwrapit (pgrad2_op, rr, hh, hmov, size (z), niter) 
    else   
       rr = grad2init (z)
       call igrad2_init (size (z, 1), size (z, 2))
       call unwrapit (igrad2_op, rr, hh, hmov, size (z), niter) 
    end if

  contains

    subroutine unwrapit (oper, r, h, hmov, n, niter)  
      interface 
         function oper (adj, add, x, dat) result (stat)
           integer 		:: stat
           logical, intent (in)	:: adj, add
           real, dimension (:)	:: x, dat
         end function oper
      end interface
      integer, intent (in)       :: n, niter
      real, dimension (n * 2)    :: r
      real, dimension (n)        :: h
      real, dimension (n, niter) :: hmov

      call cgstep_init ()
      call solver (oper, cgstep, h, r, niter, xmov=hmov, verb=.true.)
      call cgstep_close ()
    end subroutine unwrapit

  end subroutine unwrap

end module phasegrad

























